;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;; See the file LICENCE for licence information.
(in-package #:cl-user)

(defpackage #:cl-store.system
  (:use #:cl #:asdf)
  (:export #:non-required-file))


(in-package #:cl-store.system)

(defclass non-required-file (cl-source-file) ()
  (:documentation
   "File containing implementation dependent code which may or may not be there."))

(defun lisp-system-shortname ()
  #+mcl :mcl #+lispworks :lispworks #+cmu :cmucl #+clisp :clisp #+sbcl :sbcl
  #+allegro :acl #+ecl :ecl)

(defmethod component-pathname ((component non-required-file))
  (let ((pathname (call-next-method))
        (name (string-downcase (lisp-system-shortname))))
    (merge-pathnames
     (make-pathname :directory (list :relative name))
     pathname)))

(defmethod perform ((op compile-op) (component non-required-file))
  (when (probe-file (component-pathname component))
    (call-next-method)))

(defmethod perform ((op load-op) (component non-required-file))
  (when (probe-file (component-pathname component))
    (call-next-method)))

(defmethod operation-done-p ((o operation) (c non-required-file))
  (when (probe-file (component-pathname c))
    (call-next-method)))

(defsystem cl-store
  :name "CL-STORE"
  :author "Sean Ross <sdr@jhb.ucs.co.za>"
  :maintainer "Sean Ross <sdr@jhb.ucs.co.za>"
  :version "0.6"
  :description "Serialization package"
  :long-description "Portable CL Package to serialize data"
  :licence "MIT"
  :components ((:file "package")
               #+(and clisp (not mop))
               (:non-required-file "mop" :depends-on ("package"))
               (:file "utils" :depends-on ("package"))
               (:file "backends" :depends-on ("utils"))
               (:file "plumbing" :depends-on ("backends"))
               (:file "circularities" :depends-on ("plumbing"))
               (:file "default-backend" :depends-on ("circularities"))
               (:non-required-file "custom" :depends-on ("default-backend"))))

(defmethod perform :after ((o load-op) (c (eql (find-system :cl-store))))
  (provide 'cl-store))

(defmethod perform ((op test-op) (sys (eql (find-system :cl-store))))
  (oos 'load-op :cl-store-tests)
  (oos 'test-op :cl-store-tests))

(defsystem cl-store-tests
  :depends-on (rt cl-store)
  :components ((:file "tests")))

(defmethod perform ((op test-op) (sys (eql (find-system :cl-store-tests))))
  (funcall (find-symbol "RUN-TESTS" "CL-STORE-TESTS")
           (find-symbol "CL-STORE" "CL-STORE")))

;; EOF
