;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;; See the file LICENCE for licence information.

(in-package :cl-store)

;; Setup special floats
(defvar +positive-infinity+ (expt most-positive-double-float 2))
(defvar +negative-infinity+ (expt most-negative-double-float 3))
(defvar +nan-float+ (/ +negative-infinity+ +negative-infinity+))

(setf *special-floats*
  (list (cons +positive-infinity+ +positive-double-infinity-code+)
        (cons +positive-infinity+ +positive-infinity-code+)
        (cons +negative-infinity+ +negative-double-infinity-code+)
        (cons +negative-infinity+ +negative-infinity-code+)
        (cons +nan-float+ +float-double-nan-code+)
        (cons +nan-float+ +float-nan-code+)))


;; Custom structure storing from Alain Picard.
(defstore-cl-store (obj structure-object stream)
  (output-type-code +structure-object-code+ stream)
  (let* ((slot-names (structure:structure-class-slot-names (class-of obj))))
    (store-object (type-of obj) stream)
    (store-object (length slot-names) stream)
    (dolist (slot-name slot-names)
      (store-object slot-name stream)
      (store-object (slot-value obj slot-name) stream))))

(defrestore-cl-store (structure-object stream)
  (let* ((class (find-class (restore-object stream)))
         (length (restore-object stream))
         (new-instance (structure::allocate-instance class)))
    (loop repeat length do
          (let ((slot-name (restore-object stream)))
            ;; slot-names are always symbols so we don't
            ;; have to worry about circularities
            (resolving-object (obj new-instance)
              (setting (slot-value obj slot-name) (restore-object stream)))))
    new-instance))

;; EOF
